<?php

require_once("lib-action/action.php");
require_once("UI/Page.php");

class ShowProducts extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if (1 == bgpc("sync")) {
			$this->processSync();
		}

		$page = new ShowProductsPage();
		$page->createControls();
		$page->description = !$this->_isReseller() ? blmsg("TRANS_SERVICE_AND_RESELLER_PLAN_LIST_DESC") : blmsg("TRANS_PLAN_LIST_DESCRIPTION");

		if (!$this->_isReseller()) {
			require_once("UI/Tabs/UIPlansScreenTabs.php");
			$screenTabs = new UIPlansScreenTabs();
			$screenTabs->setCurrent(bgpc("reseller") ? "resellerPlans" : "servicePlans");
			$page->setScreenTabs($screenTabs);
		}

		$this->smartyVars["page"] = $page;
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}

	private function processSync()
	{
		$planIDs = bgpc("id", array());
		if (empty($planIDs)) {
			return;
		}
		$errors = array();
		foreach ($planIDs as $planID) {
			try {
				$plan = Factory()->Product($planID);
				$plan->sync_tasks |= Product::SYNC_NEED_REPAIR;
				require_once("object_managers/ProductManager.php");
				ProductManager::sync($plan);
				if (!is_null($plan->sync_error)) {
					$errors[] = blmsg("TRANS_SYNC_PLAN_ERROR_MSG", array("PLAN_ID" => $plan->id, "ERROR" => $plan->sync_error));
				}
			} catch (ProductException $e) {
				$errors[] = blmsg("TRANS_SYNC_PLAN_ERROR_MSG", array("PLAN_ID" => $planID, "ERROR" => $e->getMessage()));
				if ("ShowProductDetails" == bgpc("redirectAction")) {
					billing_put_error(end($errors));
				}
			}
		}
		if ("ShowProductDetails" == bgpc("redirectAction")) {
			$plan = Factory()->Product(reset($planIDs));
			billing_put_info(blmsg($plan->isAddon() ? "TRANS_SYNC_PLAN_ADDON_SUCCESS_MSG" : "TRANS_SYNC_PLAN_SUCCESS_MSG"));
			redirect(admin_action_url("products", "plan-overview", "index", array("id" => $plan->id)));
		}
		if (empty($errors)) {
			billing_put_info(blmsg("TRANS_SYNC_PLANS_SUCCESS_MSG"));
		} else {
			billing_put_error(blmsg("TRANS_SYNC_PLANS_FAILURE_MSG", array("ERROR" => billing_safetyhtml(implode("\n", $errors)))));
		}
	}
}

class ShowProductsPage extends UIPage
{
	public function createControls()
	{
		$listClassName = !getCurrentSession()->isReseller() && bgpc("reseller") ? 'UIResellerProductList' : 'UIProductList';

		// Prepare presentation of page
		$this->title = blmsg("TRANS_SHOW_PRODUCTS");
		$this->parent_url = action_url("dashboard", "ShowDashboard");
		// Create controls
		// Tunnel Vars for selection ability (used for import clients), ConfigGroupID, ServerGroupID

		$form = $this->getDefaultForm();
		$form->addHiddenControl($hidden = UIFactory()->InputHidden("sid")); $hidden->value = bgpc("sid");
		$form->addHiddenControl($hidden = UIFactory()->InputHidden("action")); $hidden->value = bgpc("act");

		require_once("UI/List/" . $listClassName . ".php");

		if (! is_null($tunnelVars = bgpc("tunnelVars"))) {
			$this->subtitle = blmsg("TRANS_SELECT");
			$this->subtitle_icon_id = "bid-domain-templates";
			$this->subtitle_description = blmsg("TRANS_PLEASE_SELECT_THE_PRODUCT_THIS_CLIENT_HAS");

			$this->addListPanel(new $listClassName($listClassName::MODE_FOR_CLIENT_IMPORT, array(
				"serverGroupID" => bgpc("serverGroupID"),
				"configGroupID" => bgpc("configGroupID"),
				"tunnelVars" => $tunnelVars,
			)));
		}
		else if (! is_null($clientID = bgpc("clientID"))) {
			$this->subtitle = blmsg("TRANS_SELECT");
			$this->subtitle_icon_id = "bid-domain-templates";
			$this->subtitle_description = blmsg("TRANS_PLEASE_SELECT_THE_PRODUCT_FOR_NEW_CLIENTS_PACKAGE", array(
				"CLIENT_NAME" => "#{$clientID} " . Factory()->Client($clientID)->PrimaryContact->contact_fullname,
			));

			$this->addListPanel(new $listClassName($listClassName::MODE_FOR_NEW_CLIENTS_PACKAGE, array(
				"clientID" => $clientID,
			)));
		} else {
			$this->addListPanel(new $listClassName($listClassName::MODE_NONE));
		}

		parent::createControls();
	}
}
