<?php

require_once("lib-action/action.php");

class ForwardToResellerInterface extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$resellerID = (int)$_REQUEST["resellerID"];

		if(!notZero($resellerID)) {
			throw new Exception("Missing parameter 'resellerID'");
		}

		if(!getCurrentSession()->isAdmin()) {
			throw new ProductException(ProductException::E_FORBIDDEN_OPERATION, "Must be an admin to process.");
		}

		$reseller = Factory()->Reseller($resellerID);
		$resellerUsername = Factory()->Admin($reseller->admin_id)->admin_username;

		require_once("object_managers/SecurityManager.php");
		SecurityManager::pushCurrentActionIntoLog(
			SecurityManager::USER_TYPE_RESELLER,
			$resellerID,
			$resellerUsername,
			"TRANS_ACTION_LOG_REASON_LOGIN",
			blmsg(
				"TRANS_ACTION_LOG_COMMENT_ADMIN_IMPERSONATED_RESELLER",
				array(
					"ADMIN_ID" => getCurrentSession()->user_id,
					"RESELLER_ID" => $resellerID,
					"USERNAME" => $resellerUsername,
				),
				SecurityManager::getActionLogLocale()
			)
		);

		$session = Factory()->BillingSession();
		$session->user_id = $reseller->admin_id;
		$session->user_type = BillingSession::USER_TYPE_ADMIN;
		$session->real_user_id = getCurrentSession()->real_user_id;
		$session->real_user_type = getCurrentSession()->real_user_type;
		$session->session_active = 1;
		$session->update();

		redirect_url(action_url("dashboard", "ShowDashboard", array("sid" => $session->id)));

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}