<?php
if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");
require_once mb_path("/lib-tk/include/xml/xml.php");

/**
 * This Action object collects information on a todo so that that the todo may be modified in the
 * database.
 *
 * @author  Mark Garrett
 * @author  Brian Smith
 */
class EditTodo extends MB_Action {
	/**
	 * The constructor for a EditTodo action object. This merely calls the parent constructor, 
	 * and sets the string representation of this object.
	 */
	function EditTodo() {
		parent::MB_Action();
		$this->stringRep = "EditTodo";
		$this->params = array("todoID");
	}

	/**
	 * The main method of this class that executes the EditTodo action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();
		
		$todoID = ((int) $_REQUEST["todoID"]);
		if(!$todoID) {
			$this->addError(TRANS_ACTIONERROR, "No todo ID given.");
		} else {
            $vendorId = getCurrentSession()->getVendorId();
            $todo = Factory()->TodoList(array('todo_id' => $todoID, 'vendor_id' => $vendorId));
			if (empty($todo[0])) {
                $this->addError(TRANS_ACTIONERROR, "Todo not found.");
            } else {
                $todo = $todo[0];
				$this->smartyVars["todostatus"] = getStatuses("todo");

				// Get Admins data for select menu
				$adminList = Factory()->AdminList(array("admin_active" => 1));
                if ($vendorId) {
                    $this->_removeAllWithoutHimself($adminList, $vendorId);
                } else {
                    $this->_removeAllResellers($adminList);
                }
                $todoArray = array();
                foreach ($todo->getFields() as $attributeName => $fieldValue) {
                    $oldAttributeName = getDBXMLCol($attributeName);
                    $todoArray[$oldAttributeName][0] = $todo->$attributeName;
                }
				$this->smartyVars["todo"] = correctArray( $todoArray );

				$this->mergeFormState( $this->smartyVars["todo"] );
				
				$this->formDefaults["todoDateDue"] = $this->smartyVars["todo"]["todoDateDue"];
				$this->setupDateSelector("todoDateDue");

				$this->mergeFormState($this->formDefaults+$this->smartyVars['formValues']);
				
 				$this->addTemplate("todos/EditTodo");
			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

    private function _removeAllResellers($adminList)
    {
		foreach ($adminList as $admin) {
            if ($admin->isReseller()) {
                continue;
            }
            $this->smartyVars["adminsList"][$admin->id] = $admin->fullName . " - " . $admin->admin_username;
		}
    }

    private function _removeAllWithoutHimself($adminList, $vendorId)
    {
        foreach ($adminList as $admin) {
            $reseller = $admin->getReseller();
            if ($vendorId != $reseller->id) {
                continue;
            }
            $this->smartyVars["adminsList"][$admin->id] = $admin->fullName . " - " . $admin->admin_username;
        }
    }
}