<?php
require_once('lib-mbapi/include/modules/registrar/registrar.php');
require_once('lib-mbapi/include/modules/registrar/core/regru/RegRuModule.php');

$className = 'regru';
$GLOBALS['moduleInfo'][$className] = array(
    'type'			=> 'registrar',               //module type, MUST be registrar
    'status'		=> Registrar::STATUS_STABLE,  //module status, can be Registrar::STATUS_STABLE or Registrar::STATUS_TESTING
    'version'       => '1.0.0',
    'displayName'	=> 'Reg.ru',                  //module name visible in CBM
    'author'        => 'Reg.ru',
    'countryCodes' => array('RU'),
);

class regru extends Registrar
{
	const REGRU_ORG_TYPE		= 1;
	const REGRU_INDIVIDUAL_TYPE	= 2;

	private static $contractNames = array(
		self::REGRU_ORG_TYPE		=> 'TRANS_REGISTRAR_REGRU_ORG_TYPE',
		self::REGRU_INDIVIDUAL_TYPE	=> 'TRANS_REGISTRAR_REGRU_INDIVIDUAL_TYPE',
	);

	private static $translitTlds = array(
		'tj', 'com.ua', 'kiev.ua', 'pp.ua', 'com', 'net', 'org', 'biz', 'name', 'info', 'mobi', 'uk', 'cc', 'tv', 'ws', 'bz', 'me', 'us'
			);
    /**
     * Returns the module display. This includes version number and display name.
     * @return    An array of arrays containing all the display info for the module.
     */
    public function getModuleInfo()
    {
        return $GLOBALS['moduleInfo']['regru'];
    }

	/**
	 * Construct a new regru object. This function merely sets up capabilities after calling the
	 * constructor for Registrar.
	 */
	public function __construct($input)
	{
		mb_internal_encoding('UTF8');
		mb_regex_encoding('UTF8');

		// quick fix to enable IDN support across all functions
		if (isset($this->input['domainSLD']) && trim($this->input['domainSLD']) != '') {
			$this->input['domainSLD'] = $this->getPUNYCode($this->input['domainSLD']);
		}

		$this->capabilities[REGISTRAR_REGISTERDOMAIN]					= 1;
		$this->capabilities[REGISTRAR_TRANSFERDOMAIN]					= 1;
		$this->capabilities[REGISTRAR_RENEWDOMAIN]						= 1;
		$this->capabilities[REGISTRAR_GETDOMAINEXTENDEDINFO]			= 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINAVAILABILITY]			= 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINTRANSFERAVAILABILITY]	= 0;

		//Domain Management options
		$this->capabilities[REGISTRAR_GETDOMAINCONTACTDATA]				= 0;
		$this->capabilities[REGISTRAR_SETDOMAINCONTACTDATA]				= 0;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWTYPEOPTIONS]		= 0;
		$this->capabilities[REGISTRAR_SETDOMAINRENEWTYPEOPTIONS]		= 0;
		$this->capabilities[REGISTRAR_GETDOMAINREGISTRARLOCKOPTIONS]	= 0;
		$this->capabilities[REGISTRAR_SETDOMAINREGISTRARLOCKOPTIONS]	= 0;
		$this->capabilities[REGISTRAR_GETDOMAINACCESSPASSWORD]			= 0;
		$this->capabilities[REGISTRAR_SETDOMAINACCESSPASSWORD]			= 0;
		$this->capabilities[REGISTRAR_GETDOMAINDNS]						= 0;
		$this->capabilities[REGISTRAR_SETDOMAINDNS]						= 0;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWDATE]				= 0;
		$this->capabilities[REGISTRAR_LIVERENEW]						= 0;

		$this->capabilities[REGISTRAR_CHECKACCOUNTBALANCE]				= 0;
		$this->capabilities[REGISTRAR_GETTLDPRICING]					= 0;
		$this->capabilities[REGISTRAR_GETTRANSFERSTATUS]				= 0;
		$this->capabilities[REGISTRAR_GETTLDLIST]						= 0;

		$this->capabilities[REGISTRAR_TLDS] = array(
			'ru', 'su',
			'tj',
			'com.ua', 'kiev.ua',
			//'pp.ua',
			//'kz',
			'com', 'net', 'org', 'biz', 'name', 'info', 'mobi', 'uk', 'cc', 'tv', 'ws', 'bz', 'me',
			'us',
//			'in','co.in','firm.in','gen.in','ind.in',
//			'net.in','org.in','me','us','ws','es','com.es','org.es','nom.es',
//			'com.mx','bz','com.bz','net.bz',
		);

		parent::__construct($input);

		$this->capabilities[REGISTRAR_UPLOADDOCUMENTS]					= 0;
	}

    // this function registers a new domain
    public function registerDomain()
    {
        /**
         * $this->input - an array of a domain regisrtation subscription attributes. Includes the following:
         * domainSLD, domainTLD, domainYears, domainNameserver1, domainNameserver2, domainNameserver3, domainNameserver4,
         * registrant info (registrantFirstName, registrantLastName, registrantOrganizationName,
         * registrantJobTitle, registrantAddress1, registrantAddress2, registrantCity, registrantState,
         * registrantPostalCode, registrantCountry, registrantEmail, registrantPhone, registrantFax etc
         */

        /**
         * Make an appropriate request to registrar (use Curl, Soap etc)
         * using $this->UID and $this->PW ('moduleUsername','modulePassword')
         * $registrarResponse = callToRegistrar(prepareRequest($this->input));
         * initialize the arrayData variable (what params function will returns)
         * $this->arrayData = array();
         */

        /**
         * Registrar respnse processing:
         * self::ACTION_STATUS_COMPLETED - a corresponsing Business Manager event gets 'completed', the subscription gets 'active'
         * self::ACTION_STATUS_WAITING - the event still has the 'new' status, the subscription get the 'work in progress' status. The event will be executed again in 5 minutes
         * self::ACTION_STATUS_ERROR - the event gets the 'error' status, the subscription gets 'failed'
         */
		$xml = @simplexml_load_file(dirname(__FILE__).DIRECTORY_SEPARATOR.get_class($this).'.xml');

		if (!$xml) {
			$this->addError(blmsg('TRANS_REGISTRARERROR'), get_class($this).'.xml not found or parse error');
			return $this->getRegistrarReturnXML();
		}

		$form = $xml->
			xpath("/configuration/createForms/createForm/tld[@name='".$this->input['domainTLD']."']/..");

		if (count($form) != 1) {
			$this->addError(blmsg('TRANS_REGISTRARERROR'), get_class($this).'.xml structure error');
			return $this->getRegistrarReturnXML();
		}

		$form = reset($form);

		if (isset($this->input['contract_type'])) {
			$contractType = $this->input['contract_type'];
		}

		$parameterList = array();

		foreach ($form->field as $fieldItem) {

			if (isset($contractType)) {
				$itemContractType = (integer)$fieldItem['contractType'];

				if (!empty($itemContractType) && $itemContractType != $contractType) {
					continue;
				}
			}

			$requestKey	= (string)$fieldItem['id'];
			$inputKey	= $requestKey;

			$required	= (string)$fieldItem['required'] == 'true' ? true : false;
			if ($required && empty($this->input[$inputKey])) {
				// error
				$registrarResponse['errors'][] = 'Fatal error. Required field '.$requestKey.' is empty.';
				continue;
			}

			$parameterList[$requestKey] = $this->input[$inputKey];
		}

		$parameterList['domain_name']	= $this->input['domainSLD'].'.'.$this->input['domainTLD'];
		$parameterList['period']		= $this->input['domainYears'];
		$parameterList['ns0']			= $this->input['domainNameserver1'];
		$parameterList['ns1']			= $this->input['domainNameserver2'];
		$parameterList['ns2']			= $this->input['domainNameserver3'];
		$parameterList['ns3']			= $this->input['domainNameserver4'];

		if ($this->input['domainTLD'] == 'ru' || $this->input['domainTLD'] == 'su') {
			$parameterList['period'] = 1;
		}

		$regRuModule = RegRuModule::me($this->UID, $this->PW);
		$result = $regRuModule->
			domainCreate($parameterList);

		if ($result) {
			$this->arrayData = 'success';
			$this->commandStatus = self::ACTION_STATUS_COMPLETED;
		} else {
			$this->arrayData	= 'error';
			$error				= $regRuModule->getErrorText();

			if (is_array($error)) {
				foreach ($error as $errorMessage) {
					$this->addError(blmsg('TRANS_REGISTRARERROR'), $errorMessage);
				}
			} else {
				$this->addError(blmsg('TRANS_REGISTRARERROR'), $error);
			}

			$this->commandStatus = self::ACTION_STATUS_ERROR;
		}

        // the function transfer the $this->arrayData to xml and writes the data to the subscription attributes
        return $this->getRegistrarReturnXML();
    }

    /**
     * This finction is used to set the status of a domain name registration, transfer, or renewal
     * @return true/false (true if the event is completed, otherwise - false)
     */
    public function isDone()
    {
        if (self::ACTION_STATUS_WAITING == $this->commandStatus) {
            return false;
        }
        return true;
    }

	// this function performs a domain transfer
	public function transferDomain()
	{
		/**
		 *  The function is similar to the registerDomain function, the difference is the request structure
		 *	and the using of the $this->input[domainTransferKey] parameter that defines the transferability of the domain
		 */
		$xml = @simplexml_load_file(dirname(__FILE__).DIRECTORY_SEPARATOR.get_class($this).'.xml');

		if (!$xml) {
			$this->addError(blmsg('TRANS_REGISTRARERROR'), get_class($this).'.xml not found or parse error');
			return $this->getRegistrarReturnXML();
		}

		$form = $xml->
			xpath("/configuration/createForms/createForm/tld[@name='".$this->input['domainTLD']."']/..");

		if (count($form) != 1) {
			$this->addError(blmsg('TRANS_REGISTRARERROR'), get_class($this).'.xml structure error');
			return $this->getRegistrarReturnXML();
		}

		$form = reset($form);

		if (isset($this->input['contract_type'])) {
			$contractType = $this->input['contract_type'];
		}

		$parameterList = array();

		foreach ($form->field as $fieldItem) {

			if (isset($contractType)) {
				$itemContractType = (integer)$fieldItem['contractType'];

				if (!empty($itemContractType) && $itemContractType != $contractType) {
					continue;
				}
			}

			$requestKey	= (string)$fieldItem['id'];
			$inputKey	= $requestKey;

			$required	= (string)$fieldItem['required'] == 'true' ? true : false;
			if ($required && empty($this->input[$inputKey])) {
				// error
				$registrarResponse['errors'][] = 'Fatal error. Required field '.$requestKey.' is empty.';
				continue;
			}

			$parameterList[$requestKey] = $this->input[$inputKey];
		}

		$parameterList['domain_name']	= $this->input['domainSLD'].'.'.$this->input['domainTLD'];
		$parameterList['period']		= $this->input['domainYears'];
		$parameterList['ns0']			= $this->input['domainNameserver1'];
		$parameterList['ns1']			= $this->input['domainNameserver2'];
		$parameterList['ns2']			= $this->input['domainNameserver3'];
		$parameterList['ns3']			= $this->input['domainNameserver4'];

		if ($this->input['domainTLD'] == 'ru' || $this->input['domainTLD'] == 'su') {
			$parameterList['period'] = 0;
		}

		if (in_array($this->input['domainTLD'], self::$translitTlds)) {
			$parameterList['authinfo'] = $this->input['authinfo'];
		}

		$regRuModule = RegRuModule::me($this->UID, $this->PW);
		$result = $regRuModule->domainTransfer($parameterList);

		if ($result) {
			$this->arrayData = 'success';
			$this->commandStatus = self::ACTION_STATUS_COMPLETED;
		} else {
			$this->arrayData	= 'error';
			$error				= $regRuModule->getErrorText();

			if (is_array($error)) {
				foreach ($error as $errorMessage) {
					$this->addError(blmsg('TRANS_REGISTRARERROR'), $errorMessage);
				}
			} else {
				$this->addError(blmsg('TRANS_REGISTRARERROR'), $error);
			}

			$this->commandStatus = self::ACTION_STATUS_ERROR;
		}

        // the function transfer the $this->arrayData to xml and writes the data to the subscription attributes
        return $this->getRegistrarReturnXML();
	}

	// this function renews a  domain
	public function renewDomain()
	{
		// The function is similar to the registerDomain function, the difference is the request structure.
		// Самурай без меча подобен самураю с мечом, но только без меча.

		$parameterList = array();

		$parameterList['domain_name']	= $this->input['domainSLD'].'.'.$this->input['domainTLD'];
		$parameterList['period']		= $this->input['domainYears'];

		$regRuModule = RegRuModule::me($this->UID, $this->PW);
		$result = $regRuModule->domainRenew($parameterList);

		if ($result) {
			$this->arrayData		= 'success';
			$this->commandStatus	= self::ACTION_STATUS_COMPLETED;
		} else {
			$this->arrayData	= 'error';
			$error				= $regRuModule->getErrorText();

			if (is_array($error)) {
				foreach ($error as $errorMessage) {
					$this->addError(blmsg('TRANS_REGISTRARERROR'), $errorMessage);
				}
			} else {
				$this->addError(blmsg('TRANS_REGISTRARERROR'), $error);
			}

			$this->commandStatus = self::ACTION_STATUS_ERROR;
		}

        // the function transfer the $this->arrayData to xml and writes the data to the subscription attributes
        return $this->getRegistrarReturnXML();
	}


    // this function fetches a domain contact data
    public function getDomainContactData()
    {
        // Input params ($this->input):  domainSLD, domainTLD

        /**
         * Make a request to registrar (use Curl, Soap etc)
         * using $this->UID and $this->PW ('moduleUsername','modulePassword')
         * $registrarResponse = callToRegistrar(prepareRequest($this->input));
         * initialize the arrayData variable (what params function will returns)
         * $this->arrayData = array();
         */
        switch ($registrarResponse["returnCode"]) {
            case "success":
                /**
                 * Must be filled the following arrays from $registrarResponse:
                 * $this->arrayData['DomainData']['domainname'] - domain name
                 *
                 * $this->arrayData['RegistrantContactData']
                 * $this->arrayData['AuxBillingContactData']
                 * $this->arrayData['TechContactData']
                 * $this->arrayData['AdminContactData']
                 * $this->arrayData['BillingContactData']
                 * These arrays contain the following elements:
                 *  RegistrantContactData:
                 *  registrantAddress1, registrantAddress2, registrantCity, registrantCountry,
                 *	registrantEmailAddress, registrantFax, registrantFirstName, registrantJobTitle,
                 *	registrantLastName, registrantOrganizationName, registrantPhone, registrantPhoneExt,
                 *  registrantPostalCode, registrantStateProvince, registrantStateProvinceChoice,
                 *
                 *  AuxBillingContactData:
                 *	auxAddress1, auxAddress2, auxCity, auxCountry, auxEmailAddress, "auxFax,
                 *	auxFirstName, auxJobTitle, auxLastName, auxOrganizationName, auxPhone,
                 *	auxPhoneExt, auxPostalCode, auxStateProvince, auxStateProvinceChoice,
                 *
                 *  TechContactData:
                 *	techAddress1, techAddress2, techCity, techCountry, techEmailAddress,
                 *	techFax, techFirstName, techJobTitle, techLastName, techOrganizationName,
                 *	techPhone, techPhoneExt, techPostalCode", techStateProvince,
                 *	techStateProvinceChoice,
                 *
                 *  AdminContactData:
                 *	adminAddress1, adminAddress2, "adminCity, adminCountry, adminEmailAddress,
                 *	adminFax, adminFirstName, adminJobTitle, adminLastName, adminOrganizationName,
                 *	adminPhone, adminPhoneExt, adminPostalCode", adminStateProvince,
                 * 	adminStateProvinceChoice,
                 *
                 *  BillingContactData:
                 *	billingAddress1, billingAddress2, billingCity, billingFullCountry
                 *	billingCountry, billingEmailAddress, billingFax, billingFirstName,
                 *	billingJobTitle, billingLastName, billingOrganizationName, billingPhone,
                 *	billingPhoneExt, billingPostalCode, billingStateProvince, billingStateProvinceChoice,
                 *
                 */
                break;
            default:
                foreach($registrarResponse["errors"] as $error) {
                    $this->addError(blmsg("TRANS_REGISTRARERROR"), $error);
                }
                break;
        }
        return $this->getRegistrarReturnXML();
    }


    // this function sets a domain contact data
    public function setDomainContactData()
    {
		/**	Input params:
		 *  domainSLD, domainTLD
         *
		 *  registrantAddress1, registrantAddress2, registrantCity, registrantCountry,
		 *	registrantEmailAddress, registrantFax, registrantFirstName, registrantJobTitle,
		 *	registrantLastName, registrantOrganizationName, registrantPhone, registrantPhoneExt,
		 *  registrantPostalCode, registrantStateProvince, registrantStateProvinceChoice,
         *
		 *	auxAddress1, auxAddress2, auxCity, auxCountry, auxEmailAddress, "auxFax,
		 *	auxFirstName, auxJobTitle, auxLastName, auxOrganizationName, auxPhone,
		 *	auxPhoneExt, auxPostalCode, auxStateProvince, auxStateProvinceChoice,
         *
		 *	techAddress1, techAddress2, techCity, techCountry, techEmailAddress,
		 *	techFax, techFirstName, techJobTitle, techLastName, techOrganizationName,
		 *	techPhone, techPhoneExt, techPostalCode", techStateProvince,
		 *	techStateProvinceChoice,
         *
		 *	adminAddress1, adminAddress2, "adminCity, adminCountry, adminEmailAddress,
		 *	adminFax, adminFirstName, adminJobTitle, adminLastName, adminOrganizationName,
		 *	adminPhone, adminPhoneExt, adminPostalCode", adminStateProvince,
		 * 	adminStateProvinceChoice,
         *
		 *	billingAddress1, billingAddress2, billingCity, billingFullCountry
		 *	billingCountry, billingEmailAddress, billingFax, billingFirstName,
		 *	billingJobTitle, billingLastName, billingOrganizationName, billingPhone,
		 *	billingPhoneExt, billingPostalCode, billingStateProvince, billingStateProvinceChoice,
		 *
		 */

        /**
         * Make a request to registrar (use Curl, Soap etc)
         * using $this->UID and $this->PW ('moduleUsername','modulePassword')
         * $registrarResponse = callToRegistrar(prepareRequest($this->input));
         * initialize the arrayData variable (what params function will returns)
         * $this->arrayData = array();
         */
        switch ($registrarResponse["returnCode"]) {
            case "success":
                $this->arrayData = $registrarResponse;
                break;
            default:
                foreach($registrarResponse["errors"] as $error) {
                    $this->addError(blmsg("TRANS_REGISTRARERROR"), $error);
                }
                break;
        }
        return $this->getRegistrarReturnXML();
    }


    // this function fetches a domain renewal type
    public function getDomainRenewTypeOptions()
    {
        // input params: domainSLD, domainTLD

        /**
         * Make a request to registrar (use Curl, Soap etc)
         * using $this->UID and $this->PW ('moduleUsername','modulePassword')
         * $registrarResponse = callToRegistrar(prepareRequest($this->input));
         * initialize the arrayData variable (what params function will returns)
         * $this->arrayData = array();
         */
        switch ($registrarResponse["returnCode"]) {
            case "success":
                /**
                 * Save the respons data to the following array:
                 * $this->arrayData['domainRenewType'] 0 или 1
                 */
                break;
            default:
                foreach($registrarResponse["errors"] as $error) {
                    $this->addError(blmsg("TRANS_REGISTRARERROR"), $error);
                }
                break;
        }
        return $this->getRegistrarReturnXML();
    }


    // this function sets a domain renew type
    public function setDomainRenewTypeOptions()
    {
        /**
         * input params: domainSLD, domainTLD, domainRenewType
         *	The function is similar to the getdomainRenewTypeOptions() function, the difference is the request structure.
         */
    }


    // this function fetches a domain registrar lock status
    public function getDomainRegistrarLockOptions()
    {
        // input params: domainSLD, domainTLD

        /**
         * Make arequest to registrar (use Curl, Soap etc)
         * using $this->UID and $this->PW ('moduleUsername','modulePassword')
         * $registrarResponse = callToRegistrar(prepareRequest($this->input));
         * initialize the arrayData variable (what params function will returns)
         * $this->arrayData = array();
         */
        switch ($registrarResponse["returnCode"]) {
            case "success":
                /**
                 * Save the respons data to the following arrays:
                 * $this->arrayData['domainRegistrarLock'] 1 or 0
                 * $this->arrayData['domainIsLockable'] 1 or 0
                 */
                break;
            default:
                foreach($registrarResponse["errors"] as $error) {
                    $this->addError(blmsg("TRANS_REGISTRARERROR"), $error);
                }
                break;
        }
        return $this->getRegistrarReturnXML();
    }



    // this function sets a domain registrar lock status
    public function setDomainRegistrarLockOptions()
    {
        /**
         * input params: domainSLD, domainTLD, domainRegistrarLock
         *	The function is similar to the getDomainRegistrarLockOptions() function, the difference is the request structure.

         */
    }

    // this function fetches a domain access password
    public function getDomainAccessPassword()
    {
        // input params: domainSLD, domainTLD

        /**
         * Make a request to registrar (use Curl, Soap etc)
         * using $this->UID and $this->PW ('moduleUsername','modulePassword')
         * $registrarResponse = callToRegistrar(prepareRequest($this->input));
         * initialize the arrayData variable (what params function will returns)
         * $this->arrayData = array();
         */
        switch ($registrarResponse["returnCode"]) {
            case "success":
                /**
                 * Save the respons data to the following array:
                 * $this->arrayData['domainAccessPassword']
                 */
                break;
            default:
                foreach($registrarResponse["errors"] as $error) {
                    $this->addError(blmsg("TRANS_REGISTRARERROR"), $error);
                }
                break;
        }
        return $this->getRegistrarReturnXML();
    }


    // this function sets a domain access password
    public function setDomainAccessPassword()
    {
        /**
         * input params: domainSLD, domainTLD, domainAccessPassword
		 *	The function is similar to the getDomainAccessPassword() function, the difference is the request structure.
         */
    }


    // this function fetches a domain renewal information
     public function getDomainExtendedInfo()
     {
        // input params: domainSLD, domainTLD

        /**
         * Make a request to registrar (use Curl, Soap etc)
         * using $this->UID and $this->PW ('moduleUsername','modulePassword')
         * $registrarResponse = callToRegistrar(prepareRequest($this->input));
         * initialize the arrayData variable (what params function will returns)
         * $this->arrayData = array();
         */
        switch ($registrarResponse["returnCode"]) {
            case "success":
                /**
                 * Save the respons data to the following array:
                 * $this->arrayData['DomainExpDate']
                 * $this->arrayData['DomainExpTimestamp']
                 * $this->arrayData['DomainRenewMaxYears']
                 * $this->arrayData['DomainRenewDefaultYears']
                 * $this->arrayData['DomainRenewPrice']
                 */
                break;
            default:
                foreach($registrarResponse["errors"] as $error) {
                    $this->addError(blmsg("TRANS_REGISTRARERROR"), $error);
                }
                break;
        }
        return $this->getRegistrarReturnXML();
    }


    // this function fetches a domain dns info
    public function getDomainDNS()
    {
        // input params: domainSLD, domainTLD

        /**
         *	There may be two kinds on this function depending on the registrar API:
		 * 	1) The registrar returns only DNS server list
		 *	2) The registrar returns the domain DNS records
         */
        /**
         * Make a request to registrar (use Curl, Soap etc)
         * using $this->UID and $this->PW ('moduleUsername','modulePassword')
         * $registrarResponse = callToRegistrar(prepareRequest($this->input));
         * initialize the arrayData variable (what params function will returns)
         * $this->arrayData = array();
         */
        switch ($registrarResponse["returnCode"]) {
            case "success":
                /**
                 * In the first case, the function returns:
                 * $this->arrayData['listDNS'] - DNS servers array
                 * $this->arrayData['useDNS'] - 0 or 1 - indicates using registrar DNS servers
                 * In the second case it returns:
                 * $this->arrayData['host']
                 * $this->arrayData['dnsOptions']
                 *
                 * For example:
                 * $this->arrayData['dnsOptions'] = array(
                 *      'A' => 'A',
                 *      'URL' => 'URL',
                 *      'MX' => 'MX',
                 *      'MXE' => 'MXE',
                 *      'CNAME' => 'CNAME',
                 *      'FRAME' => 'FRAME',
                 *      'TXT' => 'TXT'
                 *   );
                 *  host - an array of the records ar arrays with the following elements:type, name, address
                 * )
                 */
                break;
            default:
                foreach($registrarResponse["errors"] as $error) {
                    $this->addError(blmsg("TRANS_REGISTRARERROR"), $error);
                }
                break;
        }
        return $this->getRegistrarReturnXML();
    }


    // this function sets a domain dns info
    public function setDomainDNS()
    {
        /** input params:
         * domainSLD, domainTLD
         * domainNameserver - DNS servers array
         * host - an array of the records ar arrays with the following elements:type, name, address
         */

        /**
         * Make a  request to registrar (use Curl, Soap etc)
         * using $this->UID and $this->PW ('moduleUsername','modulePassword')
         * $registrarResponse = callToRegistrar(prepareRequest($this->input));
         * initialize the arrayData variable (what params function will returns)
         * $this->arrayData = array();
         */
        switch ($registrarResponse["returnCode"]) {
            case "success":
                $this->arrayData["result"] = "TRANS_REGISTRAR_MANAGE_DNS_UPDATED";
                break;
            default:
                foreach($registrarResponse["errors"] as $error) {
                    $this->addError(blmsg("TRANS_REGISTRARERROR"), $error);
                }
                break;
        }
        return $this->getRegistrarReturnXML();
    }

	protected function getDefaultConfigOptions()
	{
		return array(
			'TRANS_REGISTRAR_MODULE_SETTINGS' => array(
				array(
					'moduleUsername',
					't',
					1,
					NULL,
					blmsg('TRANS_REGISTRAR_REGRU_LOGIN'),
					NULL,
					NULL
				),
				array(
					'modulePassword',
					'p',
					1,
					NULL,
					blmsg('TRANS_REGISTRAR_REGRU_PASSWORD'),
					NULL,
					NULL
				),
			),
			'TRANS_REGISTRAR_NS' => array(
				array('domainNameserver1', 't', 1, 'ns1.reg.ru', 'TRANS_REGISTRAR_NS1', NULL, NULL),
				array('domainNameserver2', 't', 1, 'ns2.reg.ru', 'TRANS_REGISTRAR_NS2', NULL, NULL),
				array('domainNameserver3', 't', 0, NULL, 'TRANS_REGISTRAR_NS3', NULL, NULL),
				array('domainNameserver4', 't', 0, NULL, 'TRANS_REGISTRAR_NS4', NULL, NULL),
			)
		);
		}

	/** Get form field descriptor for domain contact information (filled in Store and in Admin CP before domain buying)
	 * param string $action - (register, transfer)
	 * param string $domain_sld
	 * param string $domain_tld
	 */
	public function getRegistrantParams(
		$action, $domain_sld, $domain_tld, ClientContact $contact = NULL, $withTLDParams = false
	)
	{
		$xml = @simplexml_load_file(dirname(__FILE__).DIRECTORY_SEPARATOR.get_class($this).'.xml');

		if (!$xml)
			return parent::getRegistrantParams(
				$action, $domain_sld, $domain_tld, $contact, $withTLDParams
			);

		$form = $xml->xpath("/configuration/forms/form/tld[@name='".$domain_tld."']/..");

		if (count($form) != 1) {
			$form = $xml->xpath("/configuration/forms/form/tld[@name='default']/..");

			if (count($form) != 1) {
				return parent::getRegistrantParams(
					$action, $domain_sld, $domain_tld, $contact, $withTLDParams
				);
			}
		}

		$form = reset($form);

		$inputs = array();

		require_once('object_managers/CountryManager.php');
		$countries = array();
		foreach (CountryManager::getCountries() as $code => $name) {
			$countries[] = array($code, $name);
		}

		$contractList = array();
		foreach ($form->contractType as $contractTypeItem) {
			$showList = array();
			foreach ($contractTypeItem->show as $showItem) {
				$showList[] = (string)$showItem['id'];
			}
			$contractList[(integer)$contractTypeItem['id']] = $showList;
		}

		// this is workaround!!! empty($contact->contact_company) always true
		$companyName = '';
		if (!is_null($contact))
			$companyName = $contact->contact_company;
		// //////////////////////////////////////////////////////////////////

		if (empty($companyName))
			$contractType = self::REGRU_INDIVIDUAL_TYPE;
		else
			$contractType = self::REGRU_ORG_TYPE;

		if (count($contractList) > 1) {
			$item = array(
				'id'			=> 'contractType',
				'type'			=> 's',
				'title'			=> 'Тип',
				'default'		=> $contractType,
			);

			$items = array();

			foreach ($contractList as $contractListKey => $contractListItem) {
				$items[] = array(
					$contractListKey,
					blmsg(self::$contractNames[$contractListKey]),
					$contractListItem,
					null
				);
			}

			$item['items']	= $items;
			$inputs[] = $item;
		}

		foreach ($form->input as $inputItem) {
			$default		= '';
			$defaultString	= trim((string)$inputItem['default']);

			if (!empty($defaultString)) {
				$defaultKeyList = explode(' ', $defaultString);

				if (!(empty($defaultKeyList) || is_null($contact))) {
					$defaultList = array();

					foreach ($defaultKeyList as $key) {
						$defaultList[] = $contact->$key;
					}
					$default = implode(' ', $defaultList);
				}

				if (in_array($domain_tld, self::$translitTlds)) {
					require_once('lib-mbapi/include/modules/registrar/core/regru/RegRuReplace.php');
					$default = RegRuReplace::toTranslit($default);
				}
			}

			$description		= '';
			$itemDescription	= blmsg((string)$inputItem->description);
			$itemExample		= blmsg((string)$inputItem->example);

			if (!empty($itemDescription))
				$description .= $itemDescription;
			if (!empty($itemExample))
				$description .= '<br/><em><strong>'
					.blmsg('TRANS_REGISTRAR_REGRU_EXAMPLE')
					.':</strong> '
					.$itemExample
					.'</em>';

			if (count($contractList) > 1) {
				// Sorry :( it for .ru and .su
				$required = false;
				// ///////////////////////////
			} else {
				$required = (string)$inputItem['required'] == 'true' ? true : false;
			}

			$item = array(
				'id'			=> (string)$inputItem['id'],
				'type'			=> (string)$inputItem['type'],
				'title'			=> blmsg((string)$inputItem->title),
				'description'	=> $description,
				'required'		=> $required,
				'hidden'		=>
					isset($contractList[$contractType])
					&& !in_array((string)$inputItem['id'], $contractList[$contractType]),
				'default'		=> $default,
				//'items'		=> array()
			);

			if ('x' == $item['type']) {
				$item['default'] = '<p align="center"><strong>'.$description.'</strong></p>';
			} elseif ('s' == $item['type']) {
				$items = array();

				foreach ($inputItem->item as $itemItem) {
					$show = array();

					foreach ($itemItem->show as $showItem) {
						$show[] = (string)$showItem['id'];
					}

					if (empty($show))
						$show = null;

					$items[] = array((string)$itemItem['value'], blmsg((string)$itemItem['title']), $show, null);
				}

				$item['items']	= $items;
			} elseif ('country' == $item['type']) {
				$item['default']	= is_null($contact) ? 'RU' : $contact->Country->countries_iso_2;
				$item['type']		= 's';
				$item['items']		= $countries;
			}

			$inputs[] = $item;
		}

		if (
			'transfer' == $action
			&& $this->input['domainTLD'] != 'ru'
			&& $this->input['domainTLD'] != 'su'
		) {
            $inputs[] = array(
            	'id'			=> 'authinfo',
            	'title'			=> blmsg('TRANS_DOMAIN_TRANSFER_AUTHCODE_TITLE'),
            	'description'	=> blmsg('TRANS_DOMAIN_TRANSFER_AUTHCODE_DESC'),
            	'type'			=> 't',
            	'required'		=> 1
            );
		}

		$registrantParams = array(
			array(
				'title'			=> (string)$form['title'],
				'description'	=> (string)$form['description'],
				'inputs'		=> $inputs,
			)
		);

		return $registrantParams;
	}

	public function collectRegistrantParams($action, $domain_sld, $domain_tld, ClientContact $contact = NULL)
	{
		$collectedParams = parent::collectRegistrantParams($action, $domain_sld, $domain_tld, $contact);

		$xml = @simplexml_load_file(dirname(__FILE__).DIRECTORY_SEPARATOR.get_class($this).'.xml');

		if (!$xml) {
			throw new ProductException(ProductException::E_FAIL_OPERATION, get_class($this).'.xml not found or parse error');
		}

		$form = $xml->xpath("/configuration/createForms/createForm/tld[@name='".$domain_tld."']/..");

		if (count($form) != 1) {
			throw new ProductException(ProductException::E_FAIL_OPERATION, get_class($this).'.xml structure error');
		}

		$form = reset($form);

		if (isset($collectedParams['data']['contractType'])) {
			$contractType = $collectedParams['data']['contractType'];
		}

		$messageList = array();
		require_once('lib-mbapi/include/modules/registrar/core/regru/RegRuValidate.php');
		require_once('lib-mbapi/include/modules/registrar/core/regru/RegRuReplace.php');
		$regRuValidate = new RegRuValidate;

		foreach ($form->field as $fieldItem) {
			$itemContractType	= (integer)$fieldItem['contractType'];
			$requestKey			= (string)$fieldItem['id'];
			$inputKey			= $requestKey;

			if (isset($contractType) && $itemContractType && ($contractType != $itemContractType)) {
				unset($collectedParams['data'][$inputKey]);
				continue;
			}

			$itemRequired	= (string)$fieldItem['required'] == 'true' ? true : false;
			$type			= (string)$fieldItem['type'];
			$min			= (integer)$fieldItem['min'];
			$max			= (integer)$fieldItem['max'];

			$required		= $itemRequired && empty($collectedParams['data'][$inputKey]);

			if (isset($contractType)) {
				$required = $required && ($contractType == $itemContractType || !$itemContractType);
			}

			$message = null;
			if ($required) {
				$message = blmsg('TRANS_REGISTRAR_REGRU_ERROR_MISSED');
			} else {
				if (
					$value = RegRuReplace::replace(
						$type,
						$collectedParams['data'][$inputKey]
					)
				) {
					$collectedParams['data'][$inputKey] = $value;
				}

				if (!empty($collectedParams['data'][$inputKey])) {
					if (
						$min
						&& mb_strlen($collectedParams['data'][$inputKey]) < $min
					) {
						$message = blmsg('TRANS_REGISTRAR_REGRU_ERROR_MIN').$min;
					} elseif (
						$max
						&& mb_strlen($collectedParams['data'][$inputKey]) > $max
					) {
						$message = blmsg('TRANS_REGISTRAR_REGRU_ERROR_MAX').$max;
					} elseif ($type) {
						if (!$regRuValidate->validate($type, $collectedParams['data'][$inputKey])) {
							$message = blmsg('TRANS_REGISTRAR_REGRU_ERROR_WRONG');
						}
					}
				}
			}

			if (!empty($message))
				$messageList[$inputKey] = $message;
		}

		foreach ($collectedParams['form_descriptor'] as &$params) {
			foreach ($params['inputs'] as &$input) {
//				if (isset($collectedParams['data'][$input['id']]))
//					$input['default'] = $collectedParams['data'][$input['id']];

				if (isset($messageList[$input['id']])) {
					$input['error_msg'] =
					$collectedParams['errors_list'][$input['id']] =
						$messageList[$input['id']];
				} elseif (
					isset($input['error_msg'])
					&& isset($collectedParams['errors_list'][$input['id']])
				) {
					unset($input['error_msg'], $collectedParams['errors_list'][$input['id']]);
				}
			}
		}

		return $collectedParams;
	}
}
